# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Spyder Project Contributors

# Licensed under the terms of the MIT License
# (see spyder/__init__.py for details)
# -----------------------------------------------------------------------------
"""
Text encoding utilities, text file I/O.

Functions 'get_coding', 'decode', 'encode' and 'to_unicode' come from Eric4
source code (Utilities/__init___.py) Copyright © 2003-2009 Detlev Offenbach
"""

# yapf: disable


# Standard library imports
from codecs import BOM_UTF8, BOM_UTF16, BOM_UTF32
import locale
import os
import re
import sys

# Third party imports
from chardet.universaldetector import UniversalDetector

# Local imports
from anaconda_navigator.external.binaryornot.check import is_binary
from anaconda_navigator.utils.py3compat import (is_binary_string, is_string,
                                                is_unicode, to_text_string)


# yapf: enable

PREFERRED_ENCODING = locale.getpreferredencoding()


def transcode(text, input=PREFERRED_ENCODING, output=PREFERRED_ENCODING):
    """Transcode a text string"""
    try:
        return text.decode("cp437").encode("cp1252")
    except UnicodeError:
        try:
            return text.decode("cp437").encode(output)
        except UnicodeError:
            return text


# -----------------------------------------------------------------------------
#  Functions for encoding and decoding bytes that come from
#  the *file system*.
# -----------------------------------------------------------------------------


# The default encoding for file paths and environment variables should be set
# to match the default encoding that the OS is using.
def getfilesystemencoding():
    """
    Query the filesystem for the encoding used to encode filenames and envvars.
    """
    encoding = sys.getfilesystemencoding()
    if encoding is None:
        # Must be Linux or Unix and nl_langinfo(CODESET) failed.
        encoding = PREFERRED_ENCODING
    return encoding


FS_ENCODING = getfilesystemencoding()


def to_unicode_from_fs(string):
    """
    Return a unicode version of string decoded using the file system encoding.
    """
    if not is_string(string):  # string is a QString
        string = to_text_string(string.toUtf8(), 'utf-8')
    else:
        if is_binary_string(string):
            try:
                unic = string.decode(FS_ENCODING)
            except (UnicodeError, TypeError):
                pass
            else:
                return unic
    return string


def to_fs_from_unicode(unic):
    """
    Return a byte string version of unc encoded using the file system encoding.
    """
    if is_unicode(unic):
        try:
            string = unic.encode(FS_ENCODING)
        except (UnicodeError, TypeError):
            pass
        else:
            return string
    return unic


# -----------------------------------------------------------------------------
#  Functions for encoding and decoding *text data* itself, usually originating
#  from or destined for the *contents* of a file.
# -----------------------------------------------------------------------------

# Codecs for working with files and text.
CODING_RE = re.compile(r"coding[:=]\s*([-\w_.]+)")
CODECS = [
    'utf-8', 'iso8859-1', 'iso8859-15', 'ascii', 'koi8-r', 'koi8-u',
    'iso8859-2', 'iso8859-3', 'iso8859-4', 'iso8859-5', 'iso8859-6',
    'iso8859-7', 'iso8859-8', 'iso8859-9', 'iso8859-10', 'iso8859-13',
    'iso8859-14', 'latin-1', 'utf-16'
]


def get_coding(text):
    """
    Function to get the coding of a text.
    @param text text to inspect (string)
    @return coding string
    """
    for line in text.splitlines()[:2]:
        try:
            result = CODING_RE.search(to_text_string(line))
        except UnicodeDecodeError:
            # This could fail because to_text_string assume the text is
            # utf8-like and we don't know the encoding to give it to
            # to_text_string
            pass
        else:
            if result:
                codec = result.group(1)
                # sometimes we find a false encoding that can result in errors
                if codec in CODECS:
                    return codec

    # Fallback using chardet
    if is_binary_string(text):
        detector = UniversalDetector()
        for line in text.splitlines()[:2]:
            detector.feed(line)
            if detector.done:
                break

        detector.close()
        return detector.result['encoding']

    return None


def decode(text):
    """
    Function to decode a text.

    @param text text to decode (string)
    @return decoded text and encoding
    """
    try:
        if text.startswith(BOM_UTF8):
            # UTF-8 with BOM
            return to_text_string(text[len(BOM_UTF8):], 'utf-8'), 'utf-8-bom'
        elif text.startswith(BOM_UTF16):
            # UTF-16 with BOM
            return to_text_string(text[len(BOM_UTF16):], 'utf-16'), 'utf-16'
        elif text.startswith(BOM_UTF32):
            # UTF-32 with BOM
            return to_text_string(text[len(BOM_UTF32):], 'utf-32'), 'utf-32'
        coding = get_coding(text)
        if coding:
            return to_text_string(text, coding), coding
    except (UnicodeError, LookupError):
        pass
    # Assume UTF-8
    try:
        return to_text_string(text, 'utf-8'), 'utf-8-guessed'
    except (UnicodeError, LookupError):
        pass
    # Assume Latin-1 (behaviour before 3.7.1)
    return to_text_string(text, "latin-1"), 'latin-1-guessed'


def encode(text, orig_coding):
    """
    Function to encode a text.

    @param text text to encode (string).
    @param orig_coding type of the original coding (string).
    @return encoded text and encoding.
    """
    if orig_coding == 'utf-8-bom':
        return BOM_UTF8 + text.encode("utf-8"), 'utf-8-bom'

    # Try saving with original encoding
    if orig_coding:
        try:
            return text.encode(orig_coding), orig_coding
        except (UnicodeError, LookupError):
            pass

    # Try declared coding spec
    coding = get_coding(text)
    if coding:
        try:
            return text.encode(coding), coding
        except (UnicodeError, LookupError):
            raise RuntimeError("Incorrect encoding (%s)" % coding)
    if (orig_coding and orig_coding.endswith('-default') or
            orig_coding.endswith('-guessed')):
        coding = orig_coding.replace("-default", "")
        coding = orig_coding.replace("-guessed", "")
        try:
            return text.encode(coding), coding
        except (UnicodeError, LookupError):
            pass

    # Try saving as ASCII
    try:
        return text.encode('ascii'), 'ascii'
    except UnicodeError:
        pass

    # Save as UTF-8 without BOM
    return text.encode('utf-8'), 'utf-8'


def to_unicode(string):
    """Convert a string to unicode."""
    if not is_unicode(string):
        for codec in CODECS:
            try:
                unic = to_text_string(string, codec)
            except UnicodeError:
                pass
            except TypeError:
                break
            else:
                return unic
    return string


def write(text, filename, encoding='utf-8', mode='wb'):
    """
    Write 'text' to file ('filename') assuming 'encoding'.

    Return (eventually new) encoding.
    """
    text, encoding = encode(text, encoding)
    with open(filename, mode) as textfile:
        textfile.write(text)
    return encoding


def writelines(lines, filename, encoding='utf-8', mode='wb'):
    """
    Write 'lines' to file ('filename') assuming 'encoding'.

    Return (eventually new) encoding.
    """
    return write(os.linesep.join(lines), filename, encoding, mode)


def read(filename, encoding='utf-8'):
    """
    Read text from file ('filename')
    Return text and encoding
    """
    text, encoding = decode(open(filename, 'rb').read())
    return text, encoding


def readlines(filename, encoding='utf-8'):
    """
    Read lines from file ('filename').

    Return lines and encoding.
    """
    text, encoding = read(filename, encoding)
    return text.split(os.linesep), encoding


def is_text_file(filename):
    """Test if the given path is a text-like file."""
    try:
        return not is_binary(filename)
    except (OSError, IOError):
        return False


def ensure_binary(value):
    """Ensure a file is binary format. From conda/common."""
    try:
        return value.encode('utf-8')
    except AttributeError:
        # AttributeError: '<>' object has no attribute 'encode'
        # In this case assume already binary type and do nothing
        return value



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""License Manager Dialog."""

# yapf: disable

# Third party imports
from qtpy.compat import getopenfilename
from qtpy.QtCore import (QAbstractTableModel, QModelIndex,
                         QSortFilterProxyModel, Qt, Signal)
from qtpy.QtGui import QColor
from qtpy.QtWidgets import (QAbstractItemView, QHBoxLayout, QStyle,
                            QStyledItemDelegate, QTableView, QVBoxLayout)

# Local imports
from anaconda_navigator.api.anaconda_api import AnacondaAPI
from anaconda_navigator.config import LICENSE_PATH, get_home_dir
from anaconda_navigator.utils.qthelpers import qapplication
from anaconda_navigator.widgets import (ButtonLink, ButtonNormal,
                                        ButtonPrimary, LabelBase,
                                        SpacerHorizontal, SpacerVertical)
from anaconda_navigator.widgets.dialogs import (DialogBase,
                                                MessageBoxInformation,
                                                MessageBoxRemove)


# yapf: enable

# Extra data added to the license dicts to track the file it comes from
# Defined as a constant as it is used in several places so this avoidd hard
# coding a string
COL_MAP = {
    0: '__type__',
    1: 'product',
    2: 'end_date',
    3: '__status__',
    4: 'sig',
    5: LICENSE_PATH,
}

HIDDEN_COLUMNS = [LICENSE_PATH, 'sig']


class LicenseModel(QAbstractTableModel):
    """Table model for the license view."""

    def __init__(self, parent=None, licenses=None):
        """Table model for the license view."""
        super(LicenseModel, self).__init__(parent=parent)
        self._parent = parent
        self._rows = licenses if licenses else []

    @staticmethod
    def flags(index):
        """Override Qt method."""
        if index.isValid():
            return Qt.ItemFlags(Qt.ItemIsEnabled | Qt.ItemIsSelectable)

    def data(self, index, role=Qt.DisplayRole):
        """Override Qt method."""
        if index is None:
            return None
        else:
            if not index.isValid() or not 0 <= index.row() < len(self._rows):
                return None

        row = index.row()
        column = index.column()
        license_data = self._rows[row]

        if role == Qt.DisplayRole:
            data_key = COL_MAP.get(column)
            if data_key:
                return license_data.get(data_key)
        elif role == Qt.TextAlignmentRole:
            return Qt.AlignCenter

        return

    @staticmethod
    def headerData(section, orientation, role=Qt.DisplayRole):
        """Override Qt method."""
        title = COL_MAP.get(section)
        title = title.replace('__', '')
        title = title.replace('_', ' ').capitalize()

        if orientation == Qt.Horizontal:
            if role == Qt.TextAlignmentRole:
                return int(Qt.AlignHCenter | Qt.AlignVCenter)
            elif role == Qt.DisplayRole:
                return title

    def rowCount(self, index=QModelIndex()):
        """Override Qt method."""
        return len(self._rows)

    @staticmethod
    def columnCount(index=QModelIndex()):
        """Override Qt method."""
        return len(COL_MAP)

    # --- Helpers
    # -------------------------------------------------------------------------
    def row(self, rownum):
        """Return the row data."""
        return self._rows[rownum] if rownum < len(self._rows) else None

    def load_licenses(self, licenses=None):
        """(Re)Load license data."""
        self._rows = licenses if licenses else []


class BackgroundDelegate(QStyledItemDelegate):
    """
    Delegate for handling background color in table.

    QTableView CSS styling rules are too limited so in order to get an even
    styling that matches the overall look, this delegate is needed.
    """

    def __init__(self, parent=None):
        """Delegate for handling background color in table."""
        super(BackgroundDelegate, self).__init__(parent=parent)
        self._parent = parent

    def paint(self, painter, option, index):
        """Override Qt method."""
        # To draw a border on selected cells
        if option.state & QStyle.State_Selected:
            if self._parent.hasFocus():
                color = QColor('#43B02A')  # TODO: Get this from the scss
            else:
                color = QColor('#cecece')  # TODO: Get this from the scss

            painter.save()
            painter.fillRect(option.rect, color)
            painter.restore()

            # Disable the state for the super() painter method
            option.state ^= QStyle.State_Selected

        super(BackgroundDelegate, self).paint(painter, option, index)


class LicenseTableView(QTableView):
    """License table manager view."""

    sig_dropped = Signal(object)
    sig_entered = Signal()
    sig_left = Signal()

    def __init__(self, parent=None):
        """License table manager view."""
        super(LicenseTableView, self).__init__(parent=parent)
        self.setMinimumWidth(500)
        self.setMinimumHeight(200)
        self.setAcceptDrops(True)
        self.setShowGrid(False)
        self.setSortingEnabled(True)
        self.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.setAlternatingRowColors(True)
        self.setSelectionMode(QAbstractItemView.SingleSelection)
        self.verticalHeader().hide()
        self.horizontalHeader().setStretchLastSection(True)

    def focusInEvent(self, event):
        """Override Qt Method."""
        super(LicenseTableView, self).focusInEvent(event)
        self.sig_entered.emit()

    def focusOutEvent(self, event):
        """Override Qt Method."""
        super(LicenseTableView, self).focusInEvent(event)
        self.sig_left.emit()

    def dragEnterEvent(self, event):
        """Override Qt Method."""
        self.setProperty('dragin', True)
        if event.mimeData().hasUrls():
            event.accept()
        else:
            event.ignore()

    def dragLeaveEvent(self, event):
        """Override Qt Method."""
        self.setProperty('dragin', False)

    @staticmethod
    def dragMoveEvent(event):
        """Override Qt Method."""
        if event.mimeData().hasUrls():
            event.setDropAction(Qt.CopyAction)
            event.accept()
        else:
            event.ignore()

    def dropEvent(self, event):
        """Override Qt Method."""
        self.setProperty('dragin', False)
        mimedata = event.mimeData()
        if mimedata.hasUrls():
            event.setDropAction(Qt.CopyAction)
            event.accept()
            links = []
            for url in mimedata.urls():
                links.append(str(url.toLocalFile()))
            self.sig_dropped.emit(tuple(links))
        else:
            event.ignore()

    def setProperty(self, name, value):
        """Override Qt method."""
        QTableView.setProperty(self, name, value)
        self.style().unpolish(self)
        self.style().polish(self)
        self.update()


class LicenseManagerDialog(DialogBase):
    """License Manager main dialog."""

    CONTACT_LINK = 'https://support.continuum.io/'  # TODO: Centralize this?

    # Url, Sender
    sig_url_clicked = Signal(object, object)

    def __init__(self, parent=None):
        """License Manager main dialog."""
        super(LicenseManagerDialog, self).__init__(parent=parent)

        self.api = AnacondaAPI()

        # Widgets
        self.message_box = None  # For testing
        self.button_add = ButtonPrimary('Add license')
        self.button_ok = ButtonNormal('Close')
        self.button_remove = ButtonNormal('Remove license')
        self.button_contact = ButtonLink('Please contact us.')
        self.label_info = LabelBase(
            'Manage your Continuum Analytics '
            'license keys.'
        )
        self.label_contact = LabelBase('Got a problem with your license? ')
        self.proxy_model = QSortFilterProxyModel(parent=self)
        self.model = LicenseModel(parent=self)
        self.table = LicenseTableView(parent=self)
        self.delegate = BackgroundDelegate(self.table)

        # Widget setup
        self.proxy_model.setSourceModel(self.model)
        self.table.setItemDelegate(self.delegate)
        self.table.setModel(self.proxy_model)
        self.setWindowTitle('License Manager')

        # Layouts
        layout_buttons = QHBoxLayout()
        layout_buttons.addWidget(self.label_info)
        layout_buttons.addWidget(SpacerHorizontal())
        layout_buttons.addStretch()
        layout_buttons.addWidget(self.button_add)
        layout_buttons.addWidget(SpacerHorizontal())
        layout_buttons.addWidget(self.button_remove)

        layout_buttons_bottom = QHBoxLayout()
        layout_buttons_bottom.addWidget(self.label_contact)
        layout_buttons_bottom.addWidget(self.button_contact)
        layout_buttons_bottom.addStretch()
        layout_buttons_bottom.addWidget(self.button_ok)

        layout = QVBoxLayout()
        layout.addLayout(layout_buttons)
        layout.addWidget(SpacerVertical())
        layout.addWidget(self.table)
        layout.addWidget(SpacerVertical())
        layout.addWidget(SpacerVertical())
        layout.addLayout(layout_buttons_bottom)
        self.setLayout(layout)

        # Signals
        self.button_add.clicked.connect(lambda: self.add_license())
        self.button_remove.clicked.connect(self.remove_license)
        self.button_ok.clicked.connect(self.accept)
        self.button_contact.clicked.connect(
            lambda v=None: self.sig_url_clicked.
            emit(self.CONTACT_LINK, 'License Manager')
        )
        self.table.sig_dropped.connect(self.handle_drop)

        # Setup
        self.button_add.setFocus()
        self.load_licenses()

    def handle_drop(self, links):
        """Handle a drag and drop event."""
        self.api.add_license(links)
        self.load_licenses()

    def _hide_columns(self):
        """Hide columns."""
        for key, val in COL_MAP.items():
            if val in HIDDEN_COLUMNS:
                self.table.setColumnHidden(key, True)

    def add_license(self, v=None, path=None):
        """Add license file."""
        if path is None:
            filename, selected_filter = getopenfilename(
                self,
                'Select license file',
                filters='License files (*.txt)',
                basedir=get_home_dir(),
            )

            if filename:
                paths = [filename]
            else:
                paths = []
        else:
            paths = [path]

        valid_licenses, invalid_licenses = self.api.add_license(paths)

        for path in invalid_licenses:
            text = ('File: <b>"{0}"</b>'
                    '<br>is not a valid license file.').format(path)
            self.message_box = MessageBoxInformation(
                text=text, title="Invalid license file"
            )
            self.message_box.exec_()

        if valid_licenses:
            self.load_licenses()

    def remove_license(self, row=None):
        """Remove license from file."""
        if row is None:
            index = self.table.currentIndex()
        else:
            index = self.proxy_model.index(row, 0)

        model_index = self.proxy_model.mapToSource(index)
        row_data = self.model.row(model_index.row())

        if row_data:
            text = (
                'Do you want to remove license for product:<br><br>'
                '<b>{product}</b> ({issued} - {end_date})'
            )
            text = text.format(
                product=row_data.get('product'),
                end_date=row_data.get('end_date'),
                issued=row_data.get('issued')
            )
            self.message_box = MessageBoxRemove(
                title='Remove license', text=text
            )
            if self.message_box.exec_():
                self.api.remove_license(row_data)
                self.load_licenses()

    def load_licenses(self):
        """Load license files."""
        res = self.api.load_licenses()
        self.model.load_licenses(res)
        self.proxy_model.setSourceModel(self.model)
        self.table.resizeColumnsToContents()
        self._hide_columns()
        self.update_status()

    def count(self):
        """Return the number of items in the table."""
        return self.table.model().rowCount()

    def update_status(self):
        """Update visible and enabled status for widgets based on actions."""
        self.button_remove.setEnabled(bool(self.count()))


def local_test():  # pragma: no cover
    """Run local test."""
    app = qapplication()
    w = LicenseManagerDialog()
    w.update_style_sheet()
    w.show()
    app.exec_()


if __name__ == '__main__':  # pragma: no cover
    local_test()



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Tests for the license manager dialog."""

# yapf: disable

# Third party imports
from qtpy.QtCore import QMimeData, Qt, QTimer, QUrl
from qtpy.QtGui import (QDragEnterEvent, QDragLeaveEvent, QDragMoveEvent,
                        QDropEvent)
from qtpy.QtWidgets import QPushButton
import pytest

# Local imports
from anaconda_navigator.api.anaconda_api import AnacondaAPI
from anaconda_navigator.widgets.dialogs.license import LicenseManagerDialog
import anaconda_navigator.widgets.dialogs.license as license_mod
import anaconda_navigator.widgets.dialogs.tests.data as data


# yapf: enable

EXPIRED_LICENSE_PATH = data.EXPIRED_LICENSE_PATH
INVALID_LICENSE_PATH = data.INVALID_LICENSE_PATH


# --- Helpers
# -----------------------------------------------------------------------------
class MockGetOpenFileName:
    """Mock of the QtPy getopenfilename compatibility function."""

    def __init__(self, path, selected_filter=None):
        self.path = path
        self.selected_filter = selected_filter

    def __call__(self, *args, **kwargs):
        return self.path, self.selected_filter


def create_event(widget, mime_data, event_type):
    """Emulate a event on widget of type drag/drop/move etc."""
    action = Qt.CopyAction | Qt.MoveAction
    point = widget.rect().center()
    event = event_type(point, action, mime_data, Qt.LeftButton, Qt.NoModifier)
    event.acceptProposedAction()
    return event


# --- Fixtures
# -----------------------------------------------------------------------------
@pytest.fixture
def botlicensefilled(qtbot):
    # First remove all licenses
    api = AnacondaAPI()
    api.remove_all_licenses()
    widget = LicenseManagerDialog()
    qtbot.addWidget(widget)
    widget.show()
    widget.add_license(path=EXPIRED_LICENSE_PATH)
    with qtbot.waitSignal(signal=widget.accepted, timeout=1000, raising=False):
        pass
    return qtbot, widget


@pytest.fixture
def botlicense(qtbot):
    # First remove all licenses
    api = AnacondaAPI()
    api.remove_all_licenses()
    widget = LicenseManagerDialog()
    qtbot.addWidget(widget)
    widget.show()
    return qtbot, widget


# --- Tests
# -----------------------------------------------------------------------------
#def test_contact(botlicense):
#    qtbot, widget = botlicense
#    with qtbot.waitSignal(
#        signal=widget.sig_url_clicked, timeout=2000, raising=True
#    ):
#        qtbot.mouseClick(widget.button_contact, Qt.LeftButton)
#
#
#def test_lclose(botlicense):
#    qtbot, widget = botlicense
#    with qtbot.waitSignal(signal=widget.accepted, timeout=5000, raising=True):
#        qtbot.mouseClick(widget.button_ok, Qt.LeftButton)
#
#
#def test_add(botlicense):
#    qtbot, widget = botlicense
#    widget.add_license(path=EXPIRED_LICENSE_PATH)
#    with qtbot.waitSignal(signal=widget.accepted, timeout=1000, raising=False):
#        pass
#    assert bool(widget.count())
#    assert widget.count() == 5
#
#
#def test_add_dialog(botlicense):
#    getopenfilename_original = license_mod.getopenfilename
#    license_mod.getopenfilename = MockGetOpenFileName(EXPIRED_LICENSE_PATH)
#    qtbot, widget = botlicense
#    widget.add_license(path=None)
#
#    with qtbot.waitSignal(signal=widget.accepted, timeout=1000, raising=False):
#        pass
#    assert bool(widget.count())
#    assert widget.count() == 5
#    license_mod.getopenfilename = getopenfilename_original
#
#
#def test_add_dialog_2(botlicense):
#    getopenfilename_original = license_mod.getopenfilename
#    license_mod.getopenfilename = MockGetOpenFileName('')
#    qtbot, widget = botlicense
#    widget.add_license()
#
#    with qtbot.waitSignal(signal=widget.accepted, timeout=1000, raising=False):
#        pass
#    assert not bool(widget.count())
#    assert widget.count() == 0
#    license_mod.getopenfilename = getopenfilename_original
#
#
#def test_add_dialog_invalid(botlicense):
#    getopenfilename_original = license_mod.getopenfilename
#    license_mod.getopenfilename = MockGetOpenFileName(INVALID_LICENSE_PATH)
#    qtbot, widget = botlicense
#
#    def interact():
#        widget.message_box.accept()
#        assert not bool(widget.count())
#        assert widget.count() == 0
#
#    timer = QTimer()
#    timer.singleShot(2000, interact)
#    timer.start()
#    widget.add_license(path=None)
#    license_mod.getopenfilename = getopenfilename_original
#
#
#def test_remove_accept(botlicensefilled):
#    qtbot, widget = botlicensefilled
#
#    def interact():
#        qtbot.mouseClick(widget.message_box.button_remove, Qt.LeftButton)
#
#    timer = QTimer()
#    timer.singleShot(1000, interact)
#    timer.start()
#    widget.remove_license(0)
#
#    assert bool(widget.count())
#    assert widget.count() == 4
#
#
#def test_remove_reject(botlicensefilled):
#    qtbot, widget = botlicensefilled
#
#    def interact():
#        qtbot.mouseClick(widget.message_box.button_cancel, Qt.LeftButton)
#
#    timer = QTimer()
#    timer.singleShot(1000, interact)
#    timer.start()
#    widget.remove_license(0)
#
#    assert bool(widget.count())
#    assert widget.count() == 5
#
#
#def test_remove_current(botlicensefilled):
#    qtbot, widget = botlicensefilled
#    widget.table.selectRow(0)
#
#    def interact():
#        qtbot.mouseClick(widget.message_box.button_remove, Qt.LeftButton)
#
#    timer = QTimer()
#    timer.singleShot(1000, interact)
#    timer.start()
#    widget.remove_license(row=None)
#
#    assert bool(widget.count())
#    assert widget.count() == 4
#
#
#def test_add_drop(botlicense):
#    qtbot, dialog = botlicense
#    mime_no_data = QMimeData()
#    mime_data = QMimeData()
#    mime_data.setUrls([QUrl.fromLocalFile(EXPIRED_LICENSE_PATH)])
#
#    event = create_event(dialog.table, mime_data, QDropEvent)
#    dialog.table.dropEvent(event)
#
#    assert bool(dialog.count())
#    assert dialog.count() == 5
#
#    event = create_event(dialog.table, mime_no_data, QDropEvent)
#    dialog.table.dropEvent(event)
#
#    assert bool(dialog.count())
#    assert dialog.count() == 5
#
#
#def test_data_none(botlicense):
#    qtbot, widget = botlicense
#    widget.model.data(None)
#    index = widget.model.index(-1, -1)
#    widget.model.data(index)
#
#
#def test_background_color(botlicensefilled):
#    qtbot, widget = botlicensefilled
#    button = QPushButton()
#    button.show()
#    widget.table.setFocus()
#    widget.table.selectRow(0)
#    button.setFocus()
#    with qtbot.waitSignal(signal=widget.accepted, timeout=1000, raising=False):
#        pass
#
#
#def test_drag_move_events(botlicense):
#    qtbot, dialog = botlicense
#
#    mime_data = QMimeData()
#    mime_no_data = QMimeData()
#    mime_data.setUrls([QUrl.fromLocalFile(EXPIRED_LICENSE_PATH)])
#
#    event = create_event(dialog.table, mime_data, QDragEnterEvent)
#    dialog.table.dragEnterEvent(event)
#
#    event = create_event(dialog.table, mime_no_data, QDragEnterEvent)
#    dialog.table.dragEnterEvent(event)
#
#    event = create_event(dialog.table, mime_data, QDragMoveEvent)
#    dialog.table.dragMoveEvent(event)
#
#    event = create_event(dialog.table, mime_no_data, QDragMoveEvent)
#    dialog.table.dragMoveEvent(event)
#
#    event = QDragLeaveEvent()
#    dialog.table.dragLeaveEvent(event)
